Learn R Programming

xega (version 0.9.0.23)

sgXGeneMapFactory: Factory for configuring a gene-dependent geneMap function.

Description

The geneMap function depends on the gene representation and the algorithm selected.

Usage

sgXGeneMapFactory(algorithm = "sga", method = "Bin2Dec")

Value

GeneMap function for the selected algorithm from the correct package.

Arguments

algorithm

Algorithm. Available: "sga", "sgde", "sgperm", "sge", sgp". Default: "sga".

method

The GeneMap method. The choices depend on the algorithm.

Details

Methods available for the different algorithms are:

  • "sga": "Bin2Dec", "Gray2Dec", "Identity", "Permutation".

  • "sgde": "Identity".

  • "sgperm": "Identity". The gene map function is not used in the decoder.

  • "sgp": "Identity". The gene map function is not used in the decoder.

  • "sge": "Mod" or "Bucket".

  • "sgede": "Identity".

See Also

Other Configuration: sgXCrossoverFactory(), sgXDecodeGeneFactory(), sgXInitGeneFactory(), sgXMutationFactory(), sgXReplicationFactory()

Examples

Run this code
sgXGeneMapFactory(algorithm="sga", method="Bin2Dec")

Run the code above in your browser using DataLab