powered by
The geneMap function depends on the gene representation and the algorithm selected.
sgXGeneMapFactory(algorithm = "sga", method = "Bin2Dec")
GeneMap function for the selected algorithm from the correct package.
Algorithm. Available: "sga", "sgde", "sgperm", "sge", sgp". Default: "sga".
The GeneMap method. The choices depend on the algorithm.
algorithm
Methods available for the different algorithms are:
"sga": "Bin2Dec", "Gray2Dec", "Identity", "Permutation".
"sgde": "Identity".
"sgperm": "Identity". The gene map function is not used in the decoder.
"sgp": "Identity". The gene map function is not used in the decoder.
"sge": "Mod" or "Bucket".
"sgede": "Identity".
Other Configuration: sgXCrossoverFactory(), sgXDecodeGeneFactory(), sgXInitGeneFactory(), sgXMutationFactory(), sgXReplicationFactory()
sgXCrossoverFactory()
sgXDecodeGeneFactory()
sgXInitGeneFactory()
sgXMutationFactory()
sgXReplicationFactory()
sgXGeneMapFactory(algorithm="sga", method="Bin2Dec")
Run the code above in your browser using DataLab